/* -*- mode: c++; c-basic-offset: 3; -*- */
//
//    File: dac_adc_ovfl.hh
//
//    Test 
//
#include "DatEnv.hh"
#include "TSeries.hh"
#include <vector>
#include <iosfwd>
#include <string>

class adcMod;
class FrWriter;

//======================================  Monitor class definition
class adc_dac_ovfl : public DatEnv {
public:
    adc_dac_ovfl(int argc, const char *argv[]);
    ~adc_dac_ovfl(void);

    //----------------------------------  Print channel status information
    void printResult(std::ostream& out) const;

    void ProcessData(void);
    bool configure(std::istream& in, const std::string& file);
    void reset(void);
    void write_data(const Time& t0, Interval dT);
private:
    std::string name;
    std::string ifo;
    enum mode_flags {
	dqr_mode,
	batch_mode,
    } run_mode;
    typedef std::vector<adcMod> adc_vect;
    Time      startTime;
    Time      endTime;
    std::string file_path;
    Time      currentTime;
    Interval  stride;
    adc_vect  adc_modvec;
    int       framesout;
};

//
//    AdcMod Class
//
//    The daqmodule class carries the status of each channel to be followed
//    by the bit tester.
//
class adcMod {
public:
    typedef uint32_t mask_type;
    typedef std::vector<std::string> str_vect;
    typedef std::vector<TSeries> tser_vect;
public:
    adcMod(const std::string& ifo, int dcu, int unit);
    ~adcMod(void);
    mask_type alarmBits(void) const;
    void activeMask(mask_type mask);
    void begin_stride(void);
    const TSeries& get_record(void) const;
    void process(DaccAPI& in);
    void reset(void);
    void reset_stride(void);
    void selectMask(mask_type mask);
    void setup(DaccAPI& in);
    void set_channel(size_t id, const std::string& chan);
    void set_channel_template(const std::string& chan);
    void set_signal_template(const std::string& name);
    void set_debug(int lvl);
    void set_sigal(size_t id, const std::string& name);
    const std::string& signal_name(size_t id) const;
    void write(FrWriter& w, const Time& t0, Interval dt);

public:
    std::string ifoid;
    int         dcuid;
    int         unitid;
    std::string chan_template;
    std::string signal_template;
    mask_type   active_mask;
    mask_type   select_mask;
    int         chan0_offset;
    int         debug;
    str_vect    channels;
    str_vect    signals;
    TSeries     record;
    mask_type   alarm_flags;
    Time        tCurrent;
    std::vector<int> prev;
};

inline const TSeries&
adcMod::get_record(void) const {
    return record;
}

inline adcMod::mask_type
adcMod::alarmBits(void) const {
    return alarm_flags & select_mask;
}

inline const std::string&
adcMod::signal_name(size_t id) const {
    static const std::string ermsg("*no signal name*");
    if (id >= signals.size()) return ermsg;
    return signals[id];
}
